<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Constraint.php 3335 2005-06-10 11:41:30Z andrew $
*/

/**
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     James Floyd <james@m3.net>
 */
class MAX_Maintenance_Priority_Constraint
{   
   /**
    * logical operator, and, or
    * @var string
    */
    var $logical;
    
   /**
    * constraint type
    * @var string
    */
    var $type;
    
   /**
    * constraint comparison, ==, !=, >=, <=, >, <
    * @var string
    */
    var $comparison;
    
   /**
    * constraint data
    * @var array
    */
    var $data = array();
    
   /**
    * order constraint should be executed 0-n
    * @var integer
    */
    var $executionOrder;
    
    
   /**
    * Factory method to return Constraint object depending
    * on data provided
    *
    * @param array $aAclProperties = Array
    *    (
    *        [ad_id] => 1
    *        [logical] => and
    *        [type] => time
    *        [comparison] => ==
    *        [data] => 1,7,18,23
    *        [executionorder] => 1
    *    )
    * @return object Constraint
    * @static 
    */
    function &factory($aAclProperties)
    {
        $dateTimeClasses = array('date', 'weekday', 'time');
        
        // only interested in Date time classes 
        $class = (in_array(strtolower($aAclProperties['type']), $dateTimeClasses)) 
                ? ucfirst($aAclProperties['type'])
                : 'Empty';
        $className = 'MAX_Maintenance_Priority_Constraint_' . $class;
        
        $file = MAX_PATH . '/lib/max/Maintenance/Priority/Constraint/' . $class . '.php';
        if (file_exists($file)) {
            require_once $file;
            if (class_exists($className)) {
                return new $className($aAclProperties);
            }
        }
    }
    
   /**
    * Delivery_Limitation abstract class
    *
    * @param array aLimitation array representing a limitation
    * <samp>
    * Array
    *    (
    *        [ad_id] => 3
    *        [logical] => and
    *        [type] => weekday
    *        [comparison] => ==
    *        [data] => 0,6
    *        [executionorder] => 0
    *    )
    * </samp>
    */
    function MAX_Maintenance_Priority_Constraint($aLimitation)
    {
        $this->logical = $aLimitation['logical'];
        $this->type = $aLimitation['type'];
        $this->comparison = $aLimitation['comparison'];
        
        // persist original value of $aOption['data'] as it may be useful later
        $this->data = explode(',',$aLimitation['data']);
        $this->executionOrder = $aLimitation['executionorder'];
        
        // Manipulate all constraints to be expressed in terms of when NOT to deliver ad
        // apose to the default, when to deliver
        $this->calculateNonDeliveryRestrictions();
    }
    
  /** 
   * Abstract method that must be overriden in sub class.
   *
   * Method should modify all delivery criteria to be expressed 
   * in terms of when not to deliver, for example:
   *
   * If ad set to deliver when date >= 12/04/2005 in terms of when not
   * to deliver will be date < 12/04/2005
   *         
   * If ad set to display when weekday == Sunday(0) the associated
   * non delivery options will be == Monday(1)...Saturday(6)
   *         
   * If ad set to display where time == 05:00, 06:00 and 07:00
   * non delivery constraints will be 00:00,01:00,02:00,03:00,04:00,08:00....23:00
   * @return void
   */
    function calculateNonDeliveryRestriction() {
        return MAX::raiseError('MAX_Maintenance_Priority_Constraint::calculateNoDeliveryRestrictions() is abstract!', 
            MAX_ERROR_NOMETHOD);
    }
    
   /**
    * Abstract method must be overriden in sub class
    *
    * Returns the number of minutes a single sub class represents
    * for example:
    * Date: 24 hours
    * Time: 1 hour
    * Weekday: 24 hours
    *
    * @todo     Is this required, can it be of use - this is specific to 
    *           date/time constraints
    */
    function minutesPerTimePeriod()
    {
        return MAX::raiseError('MAX_Maintenance_Priority_Constraint::minutesPerTimePeriod() is abstract!', 
            MAX_ERROR_NOMETHOD);
    }
    
    // 0 = active (deliver), 1 = inactive (do not deliver)
    function testInterval() {
        return MAX::raiseError('MAX_Maintenance_Priority_Constraint::testInterval() is abstract!', 
            MAX_ERROR_NOMETHOD);
    }
    
   /**
    * Method to return the appropriate operator to use when 
    * evaluating delivery constraints in terms of when NOT to deliver 
    * appose to the default set by delivery which is when TO deliver ads.
    *
    * For example the delivery condition to only deliver on a specific date
    * will be, date == 20050403, the same condition in terms of when not to 
    * deliver will be date != 20050403, operator change from == to !=
    *
    * @param string $operator operator ==, !=, >, >=, <, <=
    * @return mixed associated non delivery operator or PEAR::error object
    */
    function getNonDeliveryOperator($operator)
    {
        if ($operator == '!=') {
            return '==';
        } elseif ($operator == '==') {
            return '!=';
        } elseif ($operator == '>') {
            return '<=';
        } elseif ($operator == '>=') {
            return '<';
        } elseif ($operator == '<') {
            return '>=';
        } elseif ($operator == '<=') {
            return '>';
        }
        return MAX::raiseError('Illegal operator passed to ' . __CLASS__ . '::' . __FUNCTION__ , MAX_ERROR_INVALIDARGS);
    }
    
}

?>